package com.ozs.web.controller.accountmanagment;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ozs.common.config.BaseConfig;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.exception.base.BaseException;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.file.FileUtils;
import com.ozs.common.utils.http.HttpUtils;
import com.ozs.common.utils.poi.ExcelUtil;
import com.ozs.framework.config.ServerConfig;
import com.ozs.service.entity.BaseCameraManagement;
import com.ozs.service.entity.BaseDeviceDynamicManagement;
import com.ozs.service.entity.vo.BaseCameraManagementVo;
import com.ozs.service.entity.vo.BaseCameraResVo;
import com.ozs.service.service.BaseCameraManagementService;
import com.ozs.service.service.BaseDeviceDynamicManagementService;
import com.ozs.service.service.BaseRailwayManagementService;
import com.ozs.web.core.config.CaneraConfig;
import com.ozs.web.core.util.CameraUtil;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.http.MediaType;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 相机台账管理表 前端控制器
 * </p>
 *
 * @author ozs
 * @since 2023-02-17
 */
@RestController
@RequestMapping("/service/baseCameraManagement")
@Slf4j
public class BaseCameraManagementController extends BaseController {
    @Resource
    BaseCameraManagementService baseCameraManagementService;
    @Resource
    BaseDeviceDynamicManagementService baseDeviceDynamicManagementService;
    @Autowired
    private CaneraConfig caneraConfig;
    @Autowired
    private ServerConfig serverConfig;

    /**
     * 相机台账管理分页
     *
     * @param baseCameraManagementVo
     * @return
     */
    @ApiOperation(value = "相机台账管理分页")
    @PostMapping("/list")
    public AjaxResult selectBaseAccountManagement(@RequestBody BaseCameraManagementVo baseCameraManagementVo) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!StringUtils.isBlank(baseCameraManagementVo.getLineDir())) {
            lw.eq(BaseCameraManagement::getLineDir, baseCameraManagementVo.getLineDir());
        }
        if (!StringUtils.isBlank(baseCameraManagementVo.getRailwayName())) {
            lw.eq(BaseCameraManagement::getRailwayCode, baseCameraManagementVo.getRailwayName());
        }
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getDeptId())) {
            lw.eq(BaseCameraManagement::getDeptId, baseCameraManagementVo.getDeptId());
        }
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getBeginMile())) {
            lw.ge(BaseCameraManagement::getBeginMile, baseCameraManagementVo.getBeginMile());
        }
        if (!ObjectUtils.isEmpty(baseCameraManagementVo.getEndMile())) {
            lw.le(BaseCameraManagement::getEndMile, baseCameraManagementVo.getEndMile());
        }
        IPage<BaseCameraManagement> page = baseCameraManagementService.page(new Page<>(baseCameraManagementVo.getPageNum(), baseCameraManagementVo.getPageSize()), lw);
        return AjaxResult.success(page);
    }

    /**
     * 删除相机台账管理信息
     *
     * @param accountManagementIds
     * @return
     */
    @DeleteMapping("/{accountManagementIds}")
    @ApiOperation("删除相机台账管理信息")
    public AjaxResult removeAccountManagement(@PathVariable List<Long> accountManagementIds) {
        if (baseCameraManagementService.removeByIds(accountManagementIds)) {
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }

    /**
     * 新增相机台账管理信息
     *
     * @param baseCameraManagement
     * @return
     */
    @PostMapping("/saveAccountManagement")
    @ApiOperation("新增相机台账管理信息")
    public AjaxResult saveDistributionModule(@RequestBody BaseCameraManagement baseCameraManagement) {
        baseCameraManagement.setCreateBy(getUsername());
        BaseDeviceDynamicManagement baseDynamicManagement = new BaseDeviceDynamicManagement();
        baseDynamicManagement.setCameraCode(baseCameraManagement.getCameraCode());
        if (baseCameraManagementService.save(baseCameraManagement) && baseDeviceDynamicManagementService.save(baseDynamicManagement)) {
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }

    /**
     * 修改相机台账管理信息
     *
     * @param baseCameraManagement
     * @return
     */
    @PutMapping("/editAccountManagement")
    @ApiOperation("修改相机台账管理信息")
    public AjaxResult editAccountManagement(@RequestBody BaseCameraManagement baseCameraManagement) {
        baseCameraManagement.setUpdateBy(getUsername());
        if (baseCameraManagementService.updateById(baseCameraManagement)) {
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }

    @GetMapping(value = "/{id}")
    @ApiOperation("根据相机ID获取相机台账管理信息")
    public AjaxResult getInfo(@PathVariable Long id) {
        return AjaxResult.success(baseCameraManagementService.getById(id));
    }

    /**
     * 台账管理导入
     *
     * @param file
     * @param updateSupport
     * @return
     * @throws Exception
     */
    @ApiOperation("相机台账管理信息导入")
    @PostMapping("/importBaseAccountManagement")
    public AjaxResult importBaseAccountManagement(MultipartFile file, boolean updateSupport) throws Exception {
        ExcelUtil<BaseCameraManagement> util = new ExcelUtil<BaseCameraManagement>(BaseCameraManagement.class);
        List<BaseCameraManagement> AccountManageList = util.importExcel(file.getInputStream());
        String message = baseCameraManagementService.importBaseAccountManagement(AccountManageList, updateSupport);
        return AjaxResult.success(message);
    }

    /**
     * 导出台账管理
     *
     * @param response
     */
    @ApiOperation("导出相机台账管理信息")
    @PostMapping("/exportBaseAccountManagement")
    public void exportBaseAccountManagement(HttpServletResponse response, @RequestBody BaseCameraManagementVo baseAccountManagementVo) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!StringUtils.isBlank(baseAccountManagementVo.getLineDir())) {
            lw.eq(BaseCameraManagement::getLineDir, baseAccountManagementVo.getLineDir());
        }
        if (!StringUtils.isBlank(baseAccountManagementVo.getRailwayName())) {
            lw.eq(BaseCameraManagement::getRailwayCode, baseAccountManagementVo.getRailwayName());
        }
        if (!ObjectUtils.isEmpty(baseAccountManagementVo.getDeptId())) {
            lw.eq(BaseCameraManagement::getDeptId, baseAccountManagementVo.getDeptId());
        }
        if (!ObjectUtils.isEmpty(baseAccountManagementVo.getBeginMile())) {
            lw.ge(BaseCameraManagement::getBeginMile, baseAccountManagementVo.getBeginMile());
        }
        if (!ObjectUtils.isEmpty(baseAccountManagementVo.getEndMile())) {
            lw.le(BaseCameraManagement::getEndMile, baseAccountManagementVo.getEndMile());
        }
        List<BaseCameraManagement> list = baseCameraManagementService.list(lw);
        ExcelUtil<BaseCameraManagement> util = new ExcelUtil<>(BaseCameraManagement.class);
        util.exportExcel(response, list, "台账管理相机数据");
    }


    @ApiOperation("相机台账管理信息模板下载")
    @GetMapping("/downloaExcel")
    public void downloadZip(HttpServletResponse response) {
        try {
            InputStream resourceAsStream = this.getClass().getResourceAsStream("/template/provisionalPlan.xlsx");
            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
            FileUtils.setAttachmentResponseHeader(response, "相机台账管理信息模板.xlsx");
            FileUtils.writeBytesByInput(resourceAsStream, response.getOutputStream());
        } catch (Exception e) {
            log.error("下载文件失败", e);
        }
    }

    @ApiOperation("查询录像")
    @GetMapping("/api/records/{cameraCode}")
    public AjaxResult apiRecords(@PathVariable String cameraCode) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        lw.eq(BaseCameraManagement::getCameraCode, cameraCode);
        BaseCameraManagement one = baseCameraManagementService.getOne(lw);
        if (ObjectUtils.isEmpty(one)) {
            throw new BaseException("相机编号【" + cameraCode + "】不存在");
        }
        return new AjaxResult(200, "ok", CameraUtil.getPlayFlv(cameraCode, one.getChannel()));
    }

    @ApiOperation("播放回放")
    @PostMapping("/api/invite")
    public AjaxResult apiInvite(@Validated @RequestBody BaseCameraResVo vo) {
        LambdaQueryWrapper<BaseCameraManagement> lw = new LambdaQueryWrapper<BaseCameraManagement>();
        lw.eq(BaseCameraManagement::getCameraCode, vo.getCameraCode());
        BaseCameraManagement one = baseCameraManagementService.getOne(lw);
        if (ObjectUtils.isEmpty(one)) {
            throw new BaseException("相机编号【" + vo.getCameraCode() + "】不存在");
        }
        return success(serverConfig.getUrl() + CameraUtil.historyPlayList(one.getChannel(), vo.getStartTime(), vo.getEntTime()));
    }


    @ApiOperation("视频服务设备数量")
    @GetMapping("/totalNumberOfDevices")
    public AjaxResult totalNumberOfDevices() {
        long count = baseDeviceDynamicManagementService.count();
        LambdaQueryWrapper<BaseDeviceDynamicManagement> lw = new LambdaQueryWrapper<BaseDeviceDynamicManagement>();
        lw.eq(BaseDeviceDynamicManagement::getCameraState, 1);
        long onLine = baseDeviceDynamicManagementService.count(lw);
        LambdaQueryWrapper<BaseDeviceDynamicManagement> wrapper = new LambdaQueryWrapper<BaseDeviceDynamicManagement>();
        wrapper.eq(BaseDeviceDynamicManagement::getCameraState, 2);
        long offLine = baseDeviceDynamicManagementService.count(wrapper);
        List<Long> size = new ArrayList<>();
        size.add(count);
        size.add(onLine);
        size.add(offLine);
        return success(size);
    }

    @ApiOperation("视频服务树形结构")
    @GetMapping("/cameraTree")
    @Cacheable(value = "baseAccountManagement", key = "'cameraTree'")
    public AjaxResult cameraTree(@RequestParam(value = "name", required = false) String name) {
        return success(baseDeviceDynamicManagementService.cameraTree(name));
    }
}

